def test_login(client):
    """Test that you can successfully login."""
    assert client.is_logged_in()


def test_me(client):
    """Test that the me endpoint works."""
    assert client.me() == {
        "Email": "service+manifold@emeraldcloudlab.com",
        "EmailAddress": "service+manifold@emeraldcloudlab.com",
        "Id": "id:n0k9mG8070Mk",
        "Type": "Object.User.Emerald.Developer",
        "Username": "service+manifold",
    }



"""Tests for handling variable units."""

from pysll.models import VariableUnitValue


def test_parse_single_variable_unit():
    """220 nm."""
    mm_expression = 'Quantity[220, "Nanometers"]'
    expected_output = VariableUnitValue(220, "Nanometers")
    result = VariableUnitValue.parse_mathematica_expression(mm_expression)
    assert result == expected_output


def test_parse_double_variable_unit():
    """Testing the input: {220 nm, 25 C}"""
    mm_expression = """
        QuantityArray[
            StructuredArray`StructuredData[
                {2},
                {
                    {220, 25.},
                    {"Nanometers", "DegreesCelsius"},
                    {
                        {1}
                    }
                }
            ]
        ]"""
    expected_output = [
        VariableUnitValue(220, "Nanometers"),
        VariableUnitValue(25.0, "DegreesCelsius"),
    ]
    result = VariableUnitValue.parse_mathematica_expression(mm_expression)
    assert len(result) == len(expected_output)
    for x, y in zip(result, expected_output):
        assert x == y


def test_parse_double_double_variable_unit():
    """{{220 nm, 25 C}, {221 nm, 25 C}}"""
    mm_expression = """QuantityArray[
        StructuredArray`StructuredData[
            {2, 2},
            {
                {
                    {220, 25.},
                    {221, 25.}
                },
                {"Nanometers", "DegreesCelsius"},
                {
                    {1},
                    {2}
                }
            }
        ]
    ]"""
    expected_output = [
        [
            VariableUnitValue(220, "Nanometers"),
            VariableUnitValue(25.0, "DegreesCelsius"),
        ],
        [
            VariableUnitValue(221, "Nanometers"),
            VariableUnitValue(25.0, "DegreesCelsius"),
        ],
    ]
    result = VariableUnitValue.parse_mathematica_expression(mm_expression)
    assert len(result) == len(expected_output)
    for x, y in zip(result, expected_output):
        assert x == y


def test_parse_nested_structured_array_variable_unit():
    """{{220 nm, 25 C}}"""
    mm_expression = """StructuredArray[
        QuantityArray,
        List[1, 2],
        StructuredArray`StructuredData[
            QuantityArray,
            List[List[220, 25.0]],
            List[Nanometers, DegreesCelsius],
            List[List[1], List[2]]
        ]
    ]"""
    expected_output = [
        [
            VariableUnitValue(220, "Nanometers"),
            VariableUnitValue(25.0, "DegreesCelsius"),
        ]
    ]
    result = VariableUnitValue.parse_mathematica_expression(mm_expression)
    assert len(result) == len(expected_output)
    for x, y in zip(result, expected_output):
        assert x == y



def test_simple_search(client):
    """Test search."""
    response = client.search("Object.User.Emerald.Administrator", "")
    results = response[0]["References"]
    assert len(results) > 4


def test_search_max_results(client):
    """Test search with a max_results."""
    response = client.search("Object.User.Emerald.Administrator", "", max_results=3)
    results = response[0]["References"]
    assert len(results) < 4



from __future__ import annotations

import os
from dataclasses import dataclass

from pysll.utils import truthy


@dataclass
class FeatureFlags:
    use_numpy: bool

    @staticmethod
    def from_environment() -> FeatureFlags:
        return FeatureFlags(
            use_numpy=truthy(os.environ.get("PYSLL_USE_NUMPY")),
        )



from pysll.models import Model


def test_model_equality():
    assert Model("id:123") == Model("id:123")
    assert Model("id:123") == Model("id:123", type="Model.User")
    assert Model("id:123", type="Model.User") == Model("id:123")
    assert Model("id:123", type="Model.User") == Model("id:123", type="Model.User")

    assert Model("id:123") != Model("id:456")
    assert Model("id:123", type="Model.Container") != Model("id:123", type="Model.User")
    assert Model("id:123", type="Model.Container") != Model("id:456", type="Model.Container")
    assert Model("id:123", type="Model.Container") != Model("id:456")
    assert Model("id:123") != Model("id:456", type="Model.User")


def test_model_str():
    assert Model("id:123").sll_style_type() == "Model[id:123]"
    assert Model("id:123", type="Model").sll_style_type() == "Model[id:123]"
    assert Model("id:123", type="Container").sll_style_type() == "Model[Container, id:123]"
    assert Model("id:123", type="Model.Container").sll_style_type() == "Model[Container, id:123]"
    assert Model("id:123", type="Model.User.Emerald").sll_style_type() == "Model[User, Emerald, id:123]"
